//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: AntEvolveProgramTab.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Data;
using System.Threading;
using System.Windows.Forms;

namespace AntGP
{
	/// <summary>
	/// This control allows the user to configure the GeneticProgramOption parameters which are then
    /// used by the Evolver which uses Genetic Programming to Evolve a solution for the given food collection problem.
	/// </summary>
	public class AntEvolveProgramTab : System.Windows.Forms.UserControl
	{
		private System.Windows.Forms.CheckedListBox InstructionSetList;
		private System.Windows.Forms.Label InstructionSetLabel;
		private System.Windows.Forms.Label PopulationSizeLabel;
		private System.Windows.Forms.TextBox PopulationSizeTextBox;
		private System.Windows.Forms.TextBox TournamentSizeTextBox;
		private System.Windows.Forms.Label TournamentSizeLabel;
		private System.Windows.Forms.GroupBox InitialPopulationGroup;
		private System.Windows.Forms.TextBox PercentGreedyMutateTextBox;
		private System.Windows.Forms.Label PercentGreedyMutateLabel;
		private System.Windows.Forms.GroupBox FitnessCalculationLabel;
		private System.Windows.Forms.TextBox RemainingFoodValueTextBox;
		private System.Windows.Forms.Label RemainingFoodValueLabel;
		private System.Windows.Forms.TextBox UndiscoveredFoodValueTextBox;
		private System.Windows.Forms.Label UndiscoveredFoodValueLabel;
		private System.Windows.Forms.TextBox ProgramComplexityValueTextBox;
		private System.Windows.Forms.Label ProgramComplexityValueLabel;
		private System.Windows.Forms.TextBox SolutionSpeedValueTextBox;
		private System.Windows.Forms.Label SolutionSpeedValueLabel;
		private System.Windows.Forms.TextBox PercentMutateTextBox;
		private System.Windows.Forms.Label PercentMutateLabel;
		private System.Windows.Forms.GroupBox CrossoverGroup;
		private System.Windows.Forms.TextBox ProgramComplexityGroupingTextBox;
		private System.Windows.Forms.Label ProgramCompletixyGroupingLabel;
		private System.Windows.Forms.Button StartGPButton;
		private System.Windows.Forms.TextBox PercentLeafNodeTextBox;
		private System.Windows.Forms.Label PercentLeafNodeLabel;
		private System.Windows.Forms.GroupBox InterpreterGroup;
		private System.Windows.Forms.Label NumberOfAntsLabel;
		private System.Windows.Forms.TextBox NumberOfAntsTextBox;
		private System.Windows.Forms.Label PheromoneStrengthLabel;
		private System.Windows.Forms.TextBox PheromoneStrengthTextBox;
		private System.Windows.Forms.Label PheromoneSpreadSpeedLabel;
		private System.Windows.Forms.TextBox PheromoneSpreadSpeedTextBox;
		private System.Windows.Forms.Label WaitForHelpPercentLabel;
		private System.Windows.Forms.TextBox WaitForHelpPercentTextBox;
		private System.Windows.Forms.GroupBox CurrentBestResultGroup;
		private System.Windows.Forms.Label BestProgramFitnessLabel;
		private System.Windows.Forms.TextBox BestProgramFitnessTextBox;
		private System.Windows.Forms.Label BestProgramSizeLabel;
		private System.Windows.Forms.TextBox BestProgramSizeTextBox;
		private System.Windows.Forms.Label BestProgramTurnsLabel;
		private System.Windows.Forms.TextBox BestProgramNumberOfTurnsTextBox;
		private System.Windows.Forms.Label BestProgramFoodNotCollectedLabel;
		private System.Windows.Forms.TextBox BestProgramFoodNotCollectedTextBox;
		private System.Windows.Forms.Label BestProgramFoodNotDiscoveredLabel;
		private System.Windows.Forms.TextBox BestProgramFoodNotDiscoveredTextBox;
		private System.Windows.Forms.GroupBox CurrentAverageResultGroup;
		private System.Windows.Forms.TextBox AverageProgramFitnessTextBox;
		private System.Windows.Forms.Label AverageProgramFitnessLabel;
		private System.Windows.Forms.Label NumberOfTurnsPerAntLabel;
		private System.Windows.Forms.TextBox NumberOfTurnsPerAntTextBox;
        private System.Windows.Forms.GroupBox EvolverStatusGroup;
        private System.Windows.Forms.TextBox NumberOfIterationsTextBox;
        private System.Windows.Forms.Label NumberOfIterationsLabel;
        private System.Windows.Forms.TextBox CreatedPopulationSizeTextBox;
        private System.Windows.Forms.Label CreatedPopulationSizeLabel;
        private System.Windows.Forms.Label LoadMapLabel;
        private System.Windows.Forms.ComboBox CannedMapComboBox;
        private System.Windows.Forms.GroupBox MapGroup;
        private System.Windows.Forms.TextBox AverageProgramSizeTextBox;
        private System.Windows.Forms.Label AverageProgramSizeLabel;
        private System.Windows.Forms.Button PauseResumeButton;
        private System.Windows.Forms.TextBox MaximumProgramSizeTextBox;
        private System.Windows.Forms.Label MaximumProgramSizeLabel;
        private System.Windows.Forms.TextBox WaterRemainingValueTextBox;
        private System.Windows.Forms.Label WaterRemainingValueLabel;
        private System.Windows.Forms.Label WarningLabel;
        private System.Windows.Forms.CheckBox AutoDropFoodCheckBox;
        private System.Windows.Forms.TextBox BestProgramWaterRemainingTextBox;
        private System.Windows.Forms.Label BestProgramWaterRemainingLabel;
        private System.Windows.Forms.Label TitleLabel;
        private AntGP.SimulatorControl MapArea;

		/// <summary> 
		/// Required designer variable.
		/// </summary>
		private System.ComponentModel.Container components = null;

        /// <summary>
        /// Reference to the object that does all the work evolving the solution.
        /// Can be used to stop and restart the process.
        /// </summary>
        private Evolver evolver = null;

        /// <summary>
        /// Default Constructor.
        /// </summary>
		public AntEvolveProgramTab()
		{
			InitializeComponent();

			PopulateInstructionSetList();

            MapArea.DrawMap(GetSelectedMap(true));
		}

		/// <summary> 
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			if( disposing )
			{
				if(components != null)
				{
					components.Dispose();
				}
			}
			base.Dispose( disposing );
		}

		#region Component Designer generated code
		/// <summary> 
		/// Required method for Designer support - do not modify 
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
            this.InstructionSetList = new System.Windows.Forms.CheckedListBox();
            this.InstructionSetLabel = new System.Windows.Forms.Label();
            this.PopulationSizeLabel = new System.Windows.Forms.Label();
            this.PopulationSizeTextBox = new System.Windows.Forms.TextBox();
            this.TournamentSizeTextBox = new System.Windows.Forms.TextBox();
            this.TournamentSizeLabel = new System.Windows.Forms.Label();
            this.PercentLeafNodeTextBox = new System.Windows.Forms.TextBox();
            this.PercentLeafNodeLabel = new System.Windows.Forms.Label();
            this.InitialPopulationGroup = new System.Windows.Forms.GroupBox();
            this.MaximumProgramSizeTextBox = new System.Windows.Forms.TextBox();
            this.MaximumProgramSizeLabel = new System.Windows.Forms.Label();
            this.PercentGreedyMutateTextBox = new System.Windows.Forms.TextBox();
            this.PercentGreedyMutateLabel = new System.Windows.Forms.Label();
            this.FitnessCalculationLabel = new System.Windows.Forms.GroupBox();
            this.WaterRemainingValueTextBox = new System.Windows.Forms.TextBox();
            this.WaterRemainingValueLabel = new System.Windows.Forms.Label();
            this.ProgramComplexityGroupingTextBox = new System.Windows.Forms.TextBox();
            this.ProgramCompletixyGroupingLabel = new System.Windows.Forms.Label();
            this.SolutionSpeedValueTextBox = new System.Windows.Forms.TextBox();
            this.SolutionSpeedValueLabel = new System.Windows.Forms.Label();
            this.ProgramComplexityValueTextBox = new System.Windows.Forms.TextBox();
            this.ProgramComplexityValueLabel = new System.Windows.Forms.Label();
            this.UndiscoveredFoodValueTextBox = new System.Windows.Forms.TextBox();
            this.UndiscoveredFoodValueLabel = new System.Windows.Forms.Label();
            this.RemainingFoodValueTextBox = new System.Windows.Forms.TextBox();
            this.RemainingFoodValueLabel = new System.Windows.Forms.Label();
            this.PercentMutateTextBox = new System.Windows.Forms.TextBox();
            this.PercentMutateLabel = new System.Windows.Forms.Label();
            this.CrossoverGroup = new System.Windows.Forms.GroupBox();
            this.StartGPButton = new System.Windows.Forms.Button();
            this.InterpreterGroup = new System.Windows.Forms.GroupBox();
            this.AutoDropFoodCheckBox = new System.Windows.Forms.CheckBox();
            this.NumberOfTurnsPerAntTextBox = new System.Windows.Forms.TextBox();
            this.NumberOfTurnsPerAntLabel = new System.Windows.Forms.Label();
            this.WaitForHelpPercentTextBox = new System.Windows.Forms.TextBox();
            this.WaitForHelpPercentLabel = new System.Windows.Forms.Label();
            this.PheromoneSpreadSpeedTextBox = new System.Windows.Forms.TextBox();
            this.PheromoneSpreadSpeedLabel = new System.Windows.Forms.Label();
            this.PheromoneStrengthTextBox = new System.Windows.Forms.TextBox();
            this.PheromoneStrengthLabel = new System.Windows.Forms.Label();
            this.NumberOfAntsTextBox = new System.Windows.Forms.TextBox();
            this.NumberOfAntsLabel = new System.Windows.Forms.Label();
            this.CurrentBestResultGroup = new System.Windows.Forms.GroupBox();
            this.BestProgramWaterRemainingTextBox = new System.Windows.Forms.TextBox();
            this.BestProgramWaterRemainingLabel = new System.Windows.Forms.Label();
            this.BestProgramFoodNotDiscoveredTextBox = new System.Windows.Forms.TextBox();
            this.BestProgramFoodNotDiscoveredLabel = new System.Windows.Forms.Label();
            this.BestProgramFoodNotCollectedTextBox = new System.Windows.Forms.TextBox();
            this.BestProgramFoodNotCollectedLabel = new System.Windows.Forms.Label();
            this.BestProgramNumberOfTurnsTextBox = new System.Windows.Forms.TextBox();
            this.BestProgramTurnsLabel = new System.Windows.Forms.Label();
            this.BestProgramSizeTextBox = new System.Windows.Forms.TextBox();
            this.BestProgramSizeLabel = new System.Windows.Forms.Label();
            this.BestProgramFitnessTextBox = new System.Windows.Forms.TextBox();
            this.BestProgramFitnessLabel = new System.Windows.Forms.Label();
            this.CurrentAverageResultGroup = new System.Windows.Forms.GroupBox();
            this.AverageProgramSizeTextBox = new System.Windows.Forms.TextBox();
            this.AverageProgramSizeLabel = new System.Windows.Forms.Label();
            this.AverageProgramFitnessTextBox = new System.Windows.Forms.TextBox();
            this.AverageProgramFitnessLabel = new System.Windows.Forms.Label();
            this.LoadMapLabel = new System.Windows.Forms.Label();
            this.CannedMapComboBox = new System.Windows.Forms.ComboBox();
            this.MapGroup = new System.Windows.Forms.GroupBox();
            this.MapArea = new AntGP.SimulatorControl();
            this.PauseResumeButton = new System.Windows.Forms.Button();
            this.EvolverStatusGroup = new System.Windows.Forms.GroupBox();
            this.NumberOfIterationsTextBox = new System.Windows.Forms.TextBox();
            this.NumberOfIterationsLabel = new System.Windows.Forms.Label();
            this.CreatedPopulationSizeTextBox = new System.Windows.Forms.TextBox();
            this.CreatedPopulationSizeLabel = new System.Windows.Forms.Label();
            this.WarningLabel = new System.Windows.Forms.Label();
            this.TitleLabel = new System.Windows.Forms.Label();
            this.InitialPopulationGroup.SuspendLayout();
            this.FitnessCalculationLabel.SuspendLayout();
            this.CrossoverGroup.SuspendLayout();
            this.InterpreterGroup.SuspendLayout();
            this.CurrentBestResultGroup.SuspendLayout();
            this.CurrentAverageResultGroup.SuspendLayout();
            this.MapGroup.SuspendLayout();
            this.EvolverStatusGroup.SuspendLayout();
            this.SuspendLayout();
            // 
            // InstructionSetList
            // 
            this.InstructionSetList.CheckOnClick = true;
            this.InstructionSetList.Font = new System.Drawing.Font("Verdana", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.InstructionSetList.HorizontalScrollbar = true;
            this.InstructionSetList.Location = new System.Drawing.Point(4, 80);
            this.InstructionSetList.Name = "InstructionSetList";
            this.InstructionSetList.Size = new System.Drawing.Size(192, 532);
            this.InstructionSetList.TabIndex = 0;
            // 
            // InstructionSetLabel
            // 
            this.InstructionSetLabel.Location = new System.Drawing.Point(4, 64);
            this.InstructionSetLabel.Name = "InstructionSetLabel";
            this.InstructionSetLabel.Size = new System.Drawing.Size(192, 16);
            this.InstructionSetLabel.TabIndex = 0;
            this.InstructionSetLabel.Text = "Instruction Set";
            // 
            // PopulationSizeLabel
            // 
            this.PopulationSizeLabel.Location = new System.Drawing.Point(8, 20);
            this.PopulationSizeLabel.Name = "PopulationSizeLabel";
            this.PopulationSizeLabel.Size = new System.Drawing.Size(176, 16);
            this.PopulationSizeLabel.TabIndex = 0;
            this.PopulationSizeLabel.Text = "Population Size";
            // 
            // PopulationSizeTextBox
            // 
            this.PopulationSizeTextBox.Location = new System.Drawing.Point(8, 36);
            this.PopulationSizeTextBox.MaxLength = 5;
            this.PopulationSizeTextBox.Name = "PopulationSizeTextBox";
            this.PopulationSizeTextBox.Size = new System.Drawing.Size(176, 22);
            this.PopulationSizeTextBox.TabIndex = 0;
            this.PopulationSizeTextBox.Text = "500";
            this.PopulationSizeTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // TournamentSizeTextBox
            // 
            this.TournamentSizeTextBox.Location = new System.Drawing.Point(8, 35);
            this.TournamentSizeTextBox.MaxLength = 3;
            this.TournamentSizeTextBox.Name = "TournamentSizeTextBox";
            this.TournamentSizeTextBox.Size = new System.Drawing.Size(176, 22);
            this.TournamentSizeTextBox.TabIndex = 0;
            this.TournamentSizeTextBox.Text = "4";
            this.TournamentSizeTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // TournamentSizeLabel
            // 
            this.TournamentSizeLabel.Location = new System.Drawing.Point(8, 19);
            this.TournamentSizeLabel.Name = "TournamentSizeLabel";
            this.TournamentSizeLabel.Size = new System.Drawing.Size(176, 16);
            this.TournamentSizeLabel.TabIndex = 0;
            this.TournamentSizeLabel.Text = "Tournament Size";
            // 
            // PercentLeafNodeTextBox
            // 
            this.PercentLeafNodeTextBox.Location = new System.Drawing.Point(8, 80);
            this.PercentLeafNodeTextBox.MaxLength = 3;
            this.PercentLeafNodeTextBox.Name = "PercentLeafNodeTextBox";
            this.PercentLeafNodeTextBox.Size = new System.Drawing.Size(176, 22);
            this.PercentLeafNodeTextBox.TabIndex = 1;
            this.PercentLeafNodeTextBox.Text = "55";
            this.PercentLeafNodeTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // PercentLeafNodeLabel
            // 
            this.PercentLeafNodeLabel.Location = new System.Drawing.Point(8, 64);
            this.PercentLeafNodeLabel.Name = "PercentLeafNodeLabel";
            this.PercentLeafNodeLabel.Size = new System.Drawing.Size(176, 16);
            this.PercentLeafNodeLabel.TabIndex = 0;
            this.PercentLeafNodeLabel.Text = "Odds that a node is a leaf";
            // 
            // InitialPopulationGroup
            // 
            this.InitialPopulationGroup.Controls.Add(this.MaximumProgramSizeTextBox);
            this.InitialPopulationGroup.Controls.Add(this.MaximumProgramSizeLabel);
            this.InitialPopulationGroup.Controls.Add(this.PercentLeafNodeTextBox);
            this.InitialPopulationGroup.Controls.Add(this.PercentLeafNodeLabel);
            this.InitialPopulationGroup.Controls.Add(this.PopulationSizeLabel);
            this.InitialPopulationGroup.Controls.Add(this.PopulationSizeTextBox);
            this.InitialPopulationGroup.Location = new System.Drawing.Point(202, 8);
            this.InitialPopulationGroup.Name = "InitialPopulationGroup";
            this.InitialPopulationGroup.Size = new System.Drawing.Size(192, 152);
            this.InitialPopulationGroup.TabIndex = 1;
            this.InitialPopulationGroup.TabStop = false;
            this.InitialPopulationGroup.Text = "Initial Population";
            // 
            // MaximumProgramSizeTextBox
            // 
            this.MaximumProgramSizeTextBox.Location = new System.Drawing.Point(8, 124);
            this.MaximumProgramSizeTextBox.MaxLength = 4;
            this.MaximumProgramSizeTextBox.Name = "MaximumProgramSizeTextBox";
            this.MaximumProgramSizeTextBox.Size = new System.Drawing.Size(176, 22);
            this.MaximumProgramSizeTextBox.TabIndex = 2;
            this.MaximumProgramSizeTextBox.Text = "99";
            this.MaximumProgramSizeTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // MaximumProgramSizeLabel
            // 
            this.MaximumProgramSizeLabel.Location = new System.Drawing.Point(8, 108);
            this.MaximumProgramSizeLabel.Name = "MaximumProgramSizeLabel";
            this.MaximumProgramSizeLabel.Size = new System.Drawing.Size(176, 16);
            this.MaximumProgramSizeLabel.TabIndex = 0;
            this.MaximumProgramSizeLabel.Text = "Maximum Program Size";
            // 
            // PercentGreedyMutateTextBox
            // 
            this.PercentGreedyMutateTextBox.Location = new System.Drawing.Point(8, 124);
            this.PercentGreedyMutateTextBox.MaxLength = 3;
            this.PercentGreedyMutateTextBox.Name = "PercentGreedyMutateTextBox";
            this.PercentGreedyMutateTextBox.Size = new System.Drawing.Size(176, 22);
            this.PercentGreedyMutateTextBox.TabIndex = 2;
            this.PercentGreedyMutateTextBox.Text = "80";
            this.PercentGreedyMutateTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // PercentGreedyMutateLabel
            // 
            this.PercentGreedyMutateLabel.Location = new System.Drawing.Point(8, 108);
            this.PercentGreedyMutateLabel.Name = "PercentGreedyMutateLabel";
            this.PercentGreedyMutateLabel.Size = new System.Drawing.Size(176, 16);
            this.PercentGreedyMutateLabel.TabIndex = 0;
            this.PercentGreedyMutateLabel.Text = "Greedy Mutation %";
            // 
            // FitnessCalculationLabel
            // 
            this.FitnessCalculationLabel.Controls.Add(this.WaterRemainingValueTextBox);
            this.FitnessCalculationLabel.Controls.Add(this.WaterRemainingValueLabel);
            this.FitnessCalculationLabel.Controls.Add(this.ProgramComplexityGroupingTextBox);
            this.FitnessCalculationLabel.Controls.Add(this.ProgramCompletixyGroupingLabel);
            this.FitnessCalculationLabel.Controls.Add(this.SolutionSpeedValueTextBox);
            this.FitnessCalculationLabel.Controls.Add(this.SolutionSpeedValueLabel);
            this.FitnessCalculationLabel.Controls.Add(this.ProgramComplexityValueTextBox);
            this.FitnessCalculationLabel.Controls.Add(this.ProgramComplexityValueLabel);
            this.FitnessCalculationLabel.Controls.Add(this.UndiscoveredFoodValueTextBox);
            this.FitnessCalculationLabel.Controls.Add(this.UndiscoveredFoodValueLabel);
            this.FitnessCalculationLabel.Controls.Add(this.RemainingFoodValueTextBox);
            this.FitnessCalculationLabel.Controls.Add(this.RemainingFoodValueLabel);
            this.FitnessCalculationLabel.Location = new System.Drawing.Point(202, 332);
            this.FitnessCalculationLabel.Name = "FitnessCalculationLabel";
            this.FitnessCalculationLabel.Size = new System.Drawing.Size(192, 280);
            this.FitnessCalculationLabel.TabIndex = 3;
            this.FitnessCalculationLabel.TabStop = false;
            this.FitnessCalculationLabel.Text = "Fitness Calculations";
            // 
            // WaterRemainingValueTextBox
            // 
            this.WaterRemainingValueTextBox.Location = new System.Drawing.Point(9, 124);
            this.WaterRemainingValueTextBox.MaxLength = 10;
            this.WaterRemainingValueTextBox.Name = "WaterRemainingValueTextBox";
            this.WaterRemainingValueTextBox.Size = new System.Drawing.Size(176, 22);
            this.WaterRemainingValueTextBox.TabIndex = 2;
            this.WaterRemainingValueTextBox.Text = "1000";
            // 
            // WaterRemainingValueLabel
            // 
            this.WaterRemainingValueLabel.Location = new System.Drawing.Point(9, 108);
            this.WaterRemainingValueLabel.Name = "WaterRemainingValueLabel";
            this.WaterRemainingValueLabel.Size = new System.Drawing.Size(176, 16);
            this.WaterRemainingValueLabel.TabIndex = 5;
            this.WaterRemainingValueLabel.Text = "Remaining Water Value";
            // 
            // ProgramComplexityGroupingTextBox
            // 
            this.ProgramComplexityGroupingTextBox.Location = new System.Drawing.Point(9, 212);
            this.ProgramComplexityGroupingTextBox.MaxLength = 5;
            this.ProgramComplexityGroupingTextBox.Name = "ProgramComplexityGroupingTextBox";
            this.ProgramComplexityGroupingTextBox.Size = new System.Drawing.Size(176, 22);
            this.ProgramComplexityGroupingTextBox.TabIndex = 4;
            this.ProgramComplexityGroupingTextBox.Text = "50";
            this.ProgramComplexityGroupingTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // ProgramCompletixyGroupingLabel
            // 
            this.ProgramCompletixyGroupingLabel.Location = new System.Drawing.Point(9, 196);
            this.ProgramCompletixyGroupingLabel.Name = "ProgramCompletixyGroupingLabel";
            this.ProgramCompletixyGroupingLabel.Size = new System.Drawing.Size(176, 16);
            this.ProgramCompletixyGroupingLabel.TabIndex = 0;
            this.ProgramCompletixyGroupingLabel.Text = "Program Complex Group";
            // 
            // SolutionSpeedValueTextBox
            // 
            this.SolutionSpeedValueTextBox.Location = new System.Drawing.Point(8, 252);
            this.SolutionSpeedValueTextBox.MaxLength = 10;
            this.SolutionSpeedValueTextBox.Name = "SolutionSpeedValueTextBox";
            this.SolutionSpeedValueTextBox.Size = new System.Drawing.Size(176, 22);
            this.SolutionSpeedValueTextBox.TabIndex = 5;
            this.SolutionSpeedValueTextBox.Text = "1";
            this.SolutionSpeedValueTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // SolutionSpeedValueLabel
            // 
            this.SolutionSpeedValueLabel.Location = new System.Drawing.Point(9, 237);
            this.SolutionSpeedValueLabel.Name = "SolutionSpeedValueLabel";
            this.SolutionSpeedValueLabel.Size = new System.Drawing.Size(176, 15);
            this.SolutionSpeedValueLabel.TabIndex = 0;
            this.SolutionSpeedValueLabel.Text = "Solution Speed Value";
            // 
            // ProgramComplexityValueTextBox
            // 
            this.ProgramComplexityValueTextBox.Location = new System.Drawing.Point(9, 168);
            this.ProgramComplexityValueTextBox.MaxLength = 10;
            this.ProgramComplexityValueTextBox.Name = "ProgramComplexityValueTextBox";
            this.ProgramComplexityValueTextBox.Size = new System.Drawing.Size(176, 22);
            this.ProgramComplexityValueTextBox.TabIndex = 3;
            this.ProgramComplexityValueTextBox.Text = "100";
            this.ProgramComplexityValueTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // ProgramComplexityValueLabel
            // 
            this.ProgramComplexityValueLabel.Location = new System.Drawing.Point(9, 152);
            this.ProgramComplexityValueLabel.Name = "ProgramComplexityValueLabel";
            this.ProgramComplexityValueLabel.Size = new System.Drawing.Size(176, 16);
            this.ProgramComplexityValueLabel.TabIndex = 0;
            this.ProgramComplexityValueLabel.Text = "Program Complexity Value";
            // 
            // UndiscoveredFoodValueTextBox
            // 
            this.UndiscoveredFoodValueTextBox.Location = new System.Drawing.Point(10, 80);
            this.UndiscoveredFoodValueTextBox.MaxLength = 10;
            this.UndiscoveredFoodValueTextBox.Name = "UndiscoveredFoodValueTextBox";
            this.UndiscoveredFoodValueTextBox.Size = new System.Drawing.Size(176, 22);
            this.UndiscoveredFoodValueTextBox.TabIndex = 1;
            this.UndiscoveredFoodValueTextBox.Text = "1000000";
            this.UndiscoveredFoodValueTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // UndiscoveredFoodValueLabel
            // 
            this.UndiscoveredFoodValueLabel.Location = new System.Drawing.Point(9, 64);
            this.UndiscoveredFoodValueLabel.Name = "UndiscoveredFoodValueLabel";
            this.UndiscoveredFoodValueLabel.Size = new System.Drawing.Size(176, 16);
            this.UndiscoveredFoodValueLabel.TabIndex = 0;
            this.UndiscoveredFoodValueLabel.Text = "Undiscovered Food Value";
            // 
            // RemainingFoodValueTextBox
            // 
            this.RemainingFoodValueTextBox.Location = new System.Drawing.Point(9, 36);
            this.RemainingFoodValueTextBox.MaxLength = 10;
            this.RemainingFoodValueTextBox.Name = "RemainingFoodValueTextBox";
            this.RemainingFoodValueTextBox.Size = new System.Drawing.Size(176, 22);
            this.RemainingFoodValueTextBox.TabIndex = 0;
            this.RemainingFoodValueTextBox.Text = "100000000";
            this.RemainingFoodValueTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // RemainingFoodValueLabel
            // 
            this.RemainingFoodValueLabel.Location = new System.Drawing.Point(9, 20);
            this.RemainingFoodValueLabel.Name = "RemainingFoodValueLabel";
            this.RemainingFoodValueLabel.Size = new System.Drawing.Size(176, 16);
            this.RemainingFoodValueLabel.TabIndex = 0;
            this.RemainingFoodValueLabel.Text = "Remaining Food Value";
            // 
            // PercentMutateTextBox
            // 
            this.PercentMutateTextBox.Location = new System.Drawing.Point(8, 80);
            this.PercentMutateTextBox.MaxLength = 3;
            this.PercentMutateTextBox.Name = "PercentMutateTextBox";
            this.PercentMutateTextBox.Size = new System.Drawing.Size(176, 22);
            this.PercentMutateTextBox.TabIndex = 1;
            this.PercentMutateTextBox.Text = "100";
            this.PercentMutateTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // PercentMutateLabel
            // 
            this.PercentMutateLabel.Location = new System.Drawing.Point(8, 64);
            this.PercentMutateLabel.Name = "PercentMutateLabel";
            this.PercentMutateLabel.Size = new System.Drawing.Size(176, 16);
            this.PercentMutateLabel.TabIndex = 0;
            this.PercentMutateLabel.Text = "Mutation %";
            // 
            // CrossoverGroup
            // 
            this.CrossoverGroup.Controls.Add(this.PercentMutateLabel);
            this.CrossoverGroup.Controls.Add(this.PercentMutateTextBox);
            this.CrossoverGroup.Controls.Add(this.PercentGreedyMutateLabel);
            this.CrossoverGroup.Controls.Add(this.TournamentSizeTextBox);
            this.CrossoverGroup.Controls.Add(this.TournamentSizeLabel);
            this.CrossoverGroup.Controls.Add(this.PercentGreedyMutateTextBox);
            this.CrossoverGroup.Location = new System.Drawing.Point(202, 168);
            this.CrossoverGroup.Name = "CrossoverGroup";
            this.CrossoverGroup.Size = new System.Drawing.Size(192, 152);
            this.CrossoverGroup.TabIndex = 2;
            this.CrossoverGroup.TabStop = false;
            this.CrossoverGroup.Text = "Crossover / Mutation";
            // 
            // StartGPButton
            // 
            this.StartGPButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.StartGPButton.Location = new System.Drawing.Point(597, 3);
            this.StartGPButton.Name = "StartGPButton";
            this.StartGPButton.Size = new System.Drawing.Size(192, 48);
            this.StartGPButton.TabIndex = 15;
            this.StartGPButton.Text = "Start Genetic Program";
            this.StartGPButton.UseVisualStyleBackColor = false;
            this.StartGPButton.Click += new System.EventHandler(this.StartGPButton_Click);
            // 
            // InterpreterGroup
            // 
            this.InterpreterGroup.Controls.Add(this.AutoDropFoodCheckBox);
            this.InterpreterGroup.Controls.Add(this.NumberOfTurnsPerAntTextBox);
            this.InterpreterGroup.Controls.Add(this.NumberOfTurnsPerAntLabel);
            this.InterpreterGroup.Controls.Add(this.WaitForHelpPercentTextBox);
            this.InterpreterGroup.Controls.Add(this.WaitForHelpPercentLabel);
            this.InterpreterGroup.Controls.Add(this.PheromoneSpreadSpeedTextBox);
            this.InterpreterGroup.Controls.Add(this.PheromoneSpreadSpeedLabel);
            this.InterpreterGroup.Controls.Add(this.PheromoneStrengthTextBox);
            this.InterpreterGroup.Controls.Add(this.PheromoneStrengthLabel);
            this.InterpreterGroup.Controls.Add(this.NumberOfAntsTextBox);
            this.InterpreterGroup.Controls.Add(this.NumberOfAntsLabel);
            this.InterpreterGroup.Location = new System.Drawing.Point(400, 8);
            this.InterpreterGroup.Name = "InterpreterGroup";
            this.InterpreterGroup.Size = new System.Drawing.Size(192, 295);
            this.InterpreterGroup.TabIndex = 4;
            this.InterpreterGroup.TabStop = false;
            this.InterpreterGroup.Text = "Interpreter Options";
            // 
            // AutoDropFoodCheckBox
            // 
            this.AutoDropFoodCheckBox.Checked = true;
            this.AutoDropFoodCheckBox.CheckState = System.Windows.Forms.CheckState.Checked;
            this.AutoDropFoodCheckBox.Location = new System.Drawing.Point(6, 260);
            this.AutoDropFoodCheckBox.Name = "AutoDropFoodCheckBox";
            this.AutoDropFoodCheckBox.Size = new System.Drawing.Size(178, 33);
            this.AutoDropFoodCheckBox.TabIndex = 10;
            this.AutoDropFoodCheckBox.Text = "Automatically Drop Food when Ant is at the Nest";
            this.AutoDropFoodCheckBox.UseVisualStyleBackColor = true;
            // 
            // NumberOfTurnsPerAntTextBox
            // 
            this.NumberOfTurnsPerAntTextBox.Location = new System.Drawing.Point(8, 232);
            this.NumberOfTurnsPerAntTextBox.MaxLength = 5;
            this.NumberOfTurnsPerAntTextBox.Name = "NumberOfTurnsPerAntTextBox";
            this.NumberOfTurnsPerAntTextBox.Size = new System.Drawing.Size(176, 22);
            this.NumberOfTurnsPerAntTextBox.TabIndex = 9;
            this.NumberOfTurnsPerAntTextBox.Text = "1000";
            this.NumberOfTurnsPerAntTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // NumberOfTurnsPerAntLabel
            // 
            this.NumberOfTurnsPerAntLabel.Location = new System.Drawing.Point(8, 216);
            this.NumberOfTurnsPerAntLabel.Name = "NumberOfTurnsPerAntLabel";
            this.NumberOfTurnsPerAntLabel.Size = new System.Drawing.Size(176, 16);
            this.NumberOfTurnsPerAntLabel.TabIndex = 0;
            this.NumberOfTurnsPerAntLabel.Text = "Number of Turns per Ant";
            // 
            // WaitForHelpPercentTextBox
            // 
            this.WaitForHelpPercentTextBox.Location = new System.Drawing.Point(8, 184);
            this.WaitForHelpPercentTextBox.MaxLength = 3;
            this.WaitForHelpPercentTextBox.Name = "WaitForHelpPercentTextBox";
            this.WaitForHelpPercentTextBox.Size = new System.Drawing.Size(176, 22);
            this.WaitForHelpPercentTextBox.TabIndex = 7;
            this.WaitForHelpPercentTextBox.Text = "98";
            this.WaitForHelpPercentTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // WaitForHelpPercentLabel
            // 
            this.WaitForHelpPercentLabel.Location = new System.Drawing.Point(8, 168);
            this.WaitForHelpPercentLabel.Name = "WaitForHelpPercentLabel";
            this.WaitForHelpPercentLabel.Size = new System.Drawing.Size(176, 16);
            this.WaitForHelpPercentLabel.TabIndex = 0;
            this.WaitForHelpPercentLabel.Text = "Wait For Help Percent";
            // 
            // PheromoneSpreadSpeedTextBox
            // 
            this.PheromoneSpreadSpeedTextBox.Location = new System.Drawing.Point(8, 136);
            this.PheromoneSpreadSpeedTextBox.MaxLength = 3;
            this.PheromoneSpreadSpeedTextBox.Name = "PheromoneSpreadSpeedTextBox";
            this.PheromoneSpreadSpeedTextBox.Size = new System.Drawing.Size(176, 22);
            this.PheromoneSpreadSpeedTextBox.TabIndex = 5;
            this.PheromoneSpreadSpeedTextBox.Text = "40";
            this.PheromoneSpreadSpeedTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // PheromoneSpreadSpeedLabel
            // 
            this.PheromoneSpreadSpeedLabel.Location = new System.Drawing.Point(8, 120);
            this.PheromoneSpreadSpeedLabel.Name = "PheromoneSpreadSpeedLabel";
            this.PheromoneSpreadSpeedLabel.Size = new System.Drawing.Size(176, 16);
            this.PheromoneSpreadSpeedLabel.TabIndex = 0;
            this.PheromoneSpreadSpeedLabel.Text = "Pheromone Spread Speed";
            // 
            // PheromoneStrengthTextBox
            // 
            this.PheromoneStrengthTextBox.Location = new System.Drawing.Point(8, 88);
            this.PheromoneStrengthTextBox.MaxLength = 4;
            this.PheromoneStrengthTextBox.Name = "PheromoneStrengthTextBox";
            this.PheromoneStrengthTextBox.Size = new System.Drawing.Size(176, 22);
            this.PheromoneStrengthTextBox.TabIndex = 3;
            this.PheromoneStrengthTextBox.Text = "300";
            this.PheromoneStrengthTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // PheromoneStrengthLabel
            // 
            this.PheromoneStrengthLabel.Location = new System.Drawing.Point(8, 72);
            this.PheromoneStrengthLabel.Name = "PheromoneStrengthLabel";
            this.PheromoneStrengthLabel.Size = new System.Drawing.Size(176, 16);
            this.PheromoneStrengthLabel.TabIndex = 0;
            this.PheromoneStrengthLabel.Text = "Pheromone Strength";
            // 
            // NumberOfAntsTextBox
            // 
            this.NumberOfAntsTextBox.Location = new System.Drawing.Point(8, 40);
            this.NumberOfAntsTextBox.MaxLength = 4;
            this.NumberOfAntsTextBox.Name = "NumberOfAntsTextBox";
            this.NumberOfAntsTextBox.Size = new System.Drawing.Size(176, 22);
            this.NumberOfAntsTextBox.TabIndex = 1;
            this.NumberOfAntsTextBox.Text = "25";
            this.NumberOfAntsTextBox.Leave += new System.EventHandler(this.OptionChanged);
            // 
            // NumberOfAntsLabel
            // 
            this.NumberOfAntsLabel.Location = new System.Drawing.Point(8, 24);
            this.NumberOfAntsLabel.Name = "NumberOfAntsLabel";
            this.NumberOfAntsLabel.Size = new System.Drawing.Size(176, 16);
            this.NumberOfAntsLabel.TabIndex = 0;
            this.NumberOfAntsLabel.Text = "Number of Ants";
            // 
            // CurrentBestResultGroup
            // 
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramWaterRemainingTextBox);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramWaterRemainingLabel);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramFoodNotDiscoveredTextBox);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramFoodNotDiscoveredLabel);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramFoodNotCollectedTextBox);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramFoodNotCollectedLabel);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramNumberOfTurnsTextBox);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramTurnsLabel);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramSizeTextBox);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramSizeLabel);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramFitnessTextBox);
            this.CurrentBestResultGroup.Controls.Add(this.BestProgramFitnessLabel);
            this.CurrentBestResultGroup.Location = new System.Drawing.Point(596, 168);
            this.CurrentBestResultGroup.Name = "CurrentBestResultGroup";
            this.CurrentBestResultGroup.Size = new System.Drawing.Size(192, 284);
            this.CurrentBestResultGroup.TabIndex = 0;
            this.CurrentBestResultGroup.TabStop = false;
            this.CurrentBestResultGroup.Text = "Current Best Result of GP";
            // 
            // BestProgramWaterRemainingTextBox
            // 
            this.BestProgramWaterRemainingTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.BestProgramWaterRemainingTextBox.Location = new System.Drawing.Point(8, 168);
            this.BestProgramWaterRemainingTextBox.Name = "BestProgramWaterRemainingTextBox";
            this.BestProgramWaterRemainingTextBox.ReadOnly = true;
            this.BestProgramWaterRemainingTextBox.Size = new System.Drawing.Size(176, 22);
            this.BestProgramWaterRemainingTextBox.TabIndex = 0;
            this.BestProgramWaterRemainingTextBox.TabStop = false;
            // 
            // BestProgramWaterRemainingLabel
            // 
            this.BestProgramWaterRemainingLabel.Location = new System.Drawing.Point(8, 152);
            this.BestProgramWaterRemainingLabel.Name = "BestProgramWaterRemainingLabel";
            this.BestProgramWaterRemainingLabel.Size = new System.Drawing.Size(176, 16);
            this.BestProgramWaterRemainingLabel.TabIndex = 1;
            this.BestProgramWaterRemainingLabel.Text = "Water Remaining";
            // 
            // BestProgramFoodNotDiscoveredTextBox
            // 
            this.BestProgramFoodNotDiscoveredTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.BestProgramFoodNotDiscoveredTextBox.Location = new System.Drawing.Point(8, 256);
            this.BestProgramFoodNotDiscoveredTextBox.Name = "BestProgramFoodNotDiscoveredTextBox";
            this.BestProgramFoodNotDiscoveredTextBox.ReadOnly = true;
            this.BestProgramFoodNotDiscoveredTextBox.Size = new System.Drawing.Size(176, 22);
            this.BestProgramFoodNotDiscoveredTextBox.TabIndex = 0;
            this.BestProgramFoodNotDiscoveredTextBox.TabStop = false;
            // 
            // BestProgramFoodNotDiscoveredLabel
            // 
            this.BestProgramFoodNotDiscoveredLabel.Location = new System.Drawing.Point(8, 240);
            this.BestProgramFoodNotDiscoveredLabel.Name = "BestProgramFoodNotDiscoveredLabel";
            this.BestProgramFoodNotDiscoveredLabel.Size = new System.Drawing.Size(176, 16);
            this.BestProgramFoodNotDiscoveredLabel.TabIndex = 0;
            this.BestProgramFoodNotDiscoveredLabel.Text = "Food Not Discovered";
            // 
            // BestProgramFoodNotCollectedTextBox
            // 
            this.BestProgramFoodNotCollectedTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.BestProgramFoodNotCollectedTextBox.Location = new System.Drawing.Point(8, 212);
            this.BestProgramFoodNotCollectedTextBox.Name = "BestProgramFoodNotCollectedTextBox";
            this.BestProgramFoodNotCollectedTextBox.ReadOnly = true;
            this.BestProgramFoodNotCollectedTextBox.Size = new System.Drawing.Size(176, 22);
            this.BestProgramFoodNotCollectedTextBox.TabIndex = 0;
            this.BestProgramFoodNotCollectedTextBox.TabStop = false;
            // 
            // BestProgramFoodNotCollectedLabel
            // 
            this.BestProgramFoodNotCollectedLabel.Location = new System.Drawing.Point(8, 196);
            this.BestProgramFoodNotCollectedLabel.Name = "BestProgramFoodNotCollectedLabel";
            this.BestProgramFoodNotCollectedLabel.Size = new System.Drawing.Size(176, 16);
            this.BestProgramFoodNotCollectedLabel.TabIndex = 0;
            this.BestProgramFoodNotCollectedLabel.Text = "Food Not Collected";
            // 
            // BestProgramNumberOfTurnsTextBox
            // 
            this.BestProgramNumberOfTurnsTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.BestProgramNumberOfTurnsTextBox.Location = new System.Drawing.Point(8, 124);
            this.BestProgramNumberOfTurnsTextBox.Name = "BestProgramNumberOfTurnsTextBox";
            this.BestProgramNumberOfTurnsTextBox.ReadOnly = true;
            this.BestProgramNumberOfTurnsTextBox.Size = new System.Drawing.Size(176, 22);
            this.BestProgramNumberOfTurnsTextBox.TabIndex = 0;
            this.BestProgramNumberOfTurnsTextBox.TabStop = false;
            // 
            // BestProgramTurnsLabel
            // 
            this.BestProgramTurnsLabel.Location = new System.Drawing.Point(8, 108);
            this.BestProgramTurnsLabel.Name = "BestProgramTurnsLabel";
            this.BestProgramTurnsLabel.Size = new System.Drawing.Size(176, 16);
            this.BestProgramTurnsLabel.TabIndex = 0;
            this.BestProgramTurnsLabel.Text = "Program # of Turns";
            // 
            // BestProgramSizeTextBox
            // 
            this.BestProgramSizeTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.BestProgramSizeTextBox.Location = new System.Drawing.Point(8, 80);
            this.BestProgramSizeTextBox.Name = "BestProgramSizeTextBox";
            this.BestProgramSizeTextBox.ReadOnly = true;
            this.BestProgramSizeTextBox.Size = new System.Drawing.Size(176, 22);
            this.BestProgramSizeTextBox.TabIndex = 0;
            this.BestProgramSizeTextBox.TabStop = false;
            // 
            // BestProgramSizeLabel
            // 
            this.BestProgramSizeLabel.Location = new System.Drawing.Point(8, 64);
            this.BestProgramSizeLabel.Name = "BestProgramSizeLabel";
            this.BestProgramSizeLabel.Size = new System.Drawing.Size(176, 16);
            this.BestProgramSizeLabel.TabIndex = 0;
            this.BestProgramSizeLabel.Text = "Program Size";
            // 
            // BestProgramFitnessTextBox
            // 
            this.BestProgramFitnessTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.BestProgramFitnessTextBox.Location = new System.Drawing.Point(8, 36);
            this.BestProgramFitnessTextBox.Name = "BestProgramFitnessTextBox";
            this.BestProgramFitnessTextBox.ReadOnly = true;
            this.BestProgramFitnessTextBox.Size = new System.Drawing.Size(176, 22);
            this.BestProgramFitnessTextBox.TabIndex = 0;
            this.BestProgramFitnessTextBox.TabStop = false;
            // 
            // BestProgramFitnessLabel
            // 
            this.BestProgramFitnessLabel.Location = new System.Drawing.Point(8, 20);
            this.BestProgramFitnessLabel.Name = "BestProgramFitnessLabel";
            this.BestProgramFitnessLabel.Size = new System.Drawing.Size(176, 16);
            this.BestProgramFitnessLabel.TabIndex = 0;
            this.BestProgramFitnessLabel.Text = "Fitness";
            // 
            // CurrentAverageResultGroup
            // 
            this.CurrentAverageResultGroup.Controls.Add(this.AverageProgramSizeTextBox);
            this.CurrentAverageResultGroup.Controls.Add(this.AverageProgramSizeLabel);
            this.CurrentAverageResultGroup.Controls.Add(this.AverageProgramFitnessTextBox);
            this.CurrentAverageResultGroup.Controls.Add(this.AverageProgramFitnessLabel);
            this.CurrentAverageResultGroup.Location = new System.Drawing.Point(596, 456);
            this.CurrentAverageResultGroup.Name = "CurrentAverageResultGroup";
            this.CurrentAverageResultGroup.Size = new System.Drawing.Size(192, 107);
            this.CurrentAverageResultGroup.TabIndex = 0;
            this.CurrentAverageResultGroup.TabStop = false;
            this.CurrentAverageResultGroup.Text = "Current Average Results";
            // 
            // AverageProgramSizeTextBox
            // 
            this.AverageProgramSizeTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.AverageProgramSizeTextBox.Location = new System.Drawing.Point(8, 80);
            this.AverageProgramSizeTextBox.Name = "AverageProgramSizeTextBox";
            this.AverageProgramSizeTextBox.ReadOnly = true;
            this.AverageProgramSizeTextBox.Size = new System.Drawing.Size(176, 22);
            this.AverageProgramSizeTextBox.TabIndex = 0;
            this.AverageProgramSizeTextBox.TabStop = false;
            // 
            // AverageProgramSizeLabel
            // 
            this.AverageProgramSizeLabel.Location = new System.Drawing.Point(8, 64);
            this.AverageProgramSizeLabel.Name = "AverageProgramSizeLabel";
            this.AverageProgramSizeLabel.Size = new System.Drawing.Size(176, 16);
            this.AverageProgramSizeLabel.TabIndex = 0;
            this.AverageProgramSizeLabel.Text = "Program Size";
            // 
            // AverageProgramFitnessTextBox
            // 
            this.AverageProgramFitnessTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.AverageProgramFitnessTextBox.Location = new System.Drawing.Point(8, 36);
            this.AverageProgramFitnessTextBox.Name = "AverageProgramFitnessTextBox";
            this.AverageProgramFitnessTextBox.ReadOnly = true;
            this.AverageProgramFitnessTextBox.Size = new System.Drawing.Size(176, 22);
            this.AverageProgramFitnessTextBox.TabIndex = 0;
            this.AverageProgramFitnessTextBox.TabStop = false;
            // 
            // AverageProgramFitnessLabel
            // 
            this.AverageProgramFitnessLabel.Location = new System.Drawing.Point(8, 20);
            this.AverageProgramFitnessLabel.Name = "AverageProgramFitnessLabel";
            this.AverageProgramFitnessLabel.Size = new System.Drawing.Size(176, 16);
            this.AverageProgramFitnessLabel.TabIndex = 0;
            this.AverageProgramFitnessLabel.Text = "Fitness";
            // 
            // LoadMapLabel
            // 
            this.LoadMapLabel.Location = new System.Drawing.Point(5, 18);
            this.LoadMapLabel.Name = "LoadMapLabel";
            this.LoadMapLabel.Size = new System.Drawing.Size(176, 16);
            this.LoadMapLabel.TabIndex = 0;
            this.LoadMapLabel.Text = "Load Map";
            // 
            // CannedMapComboBox
            // 
            this.CannedMapComboBox.Font = new System.Drawing.Font("Verdana", 8.25F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.CannedMapComboBox.ItemHeight = 13;
            this.CannedMapComboBox.Items.AddRange(new object[] {
            "Use Custom Map from Tab3",
            "Three Food(1) Piles",
            "Six Weight Food(1,3,6) Piles",
            "Seven Food(6) Pieces",
            "Ten Food(9) Pieces",
            "One Food(1) Pile With River",
            "Three Food(1) Piles River Trees",
            "Six Food(5) Pieces Wall/River"});
            this.CannedMapComboBox.Location = new System.Drawing.Point(5, 34);
            this.CannedMapComboBox.Name = "CannedMapComboBox";
            this.CannedMapComboBox.Size = new System.Drawing.Size(181, 21);
            this.CannedMapComboBox.TabIndex = 30;
            this.CannedMapComboBox.Text = "Three Food(1) Piles";
            this.CannedMapComboBox.SelectedIndexChanged += new System.EventHandler(this.CannedMapComboBox_SelectedIndexChanged);
            // 
            // MapGroup
            // 
            this.MapGroup.Controls.Add(this.MapArea);
            this.MapGroup.Controls.Add(this.LoadMapLabel);
            this.MapGroup.Controls.Add(this.CannedMapComboBox);
            this.MapGroup.Location = new System.Drawing.Point(400, 312);
            this.MapGroup.Name = "MapGroup";
            this.MapGroup.Size = new System.Drawing.Size(192, 249);
            this.MapGroup.TabIndex = 5;
            this.MapGroup.TabStop = false;
            this.MapGroup.Text = "Map";
            // 
            // MapArea
            // 
            this.MapArea.BackColor = System.Drawing.Color.White;
            this.MapArea.BorderStyle = System.Windows.Forms.BorderStyle.Fixed3D;
            this.MapArea.Font = new System.Drawing.Font("Verdana", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.MapArea.ForeColor = System.Drawing.Color.Black;
            this.MapArea.Location = new System.Drawing.Point(5, 62);
            this.MapArea.Name = "MapArea";
            this.MapArea.Size = new System.Drawing.Size(181, 181);
            this.MapArea.TabIndex = 34;
            // 
            // PauseResumeButton
            // 
            this.PauseResumeButton.BackColor = System.Drawing.Color.WhiteSmoke;
            this.PauseResumeButton.Enabled = false;
            this.PauseResumeButton.Location = new System.Drawing.Point(596, 564);
            this.PauseResumeButton.Name = "PauseResumeButton";
            this.PauseResumeButton.Size = new System.Drawing.Size(192, 48);
            this.PauseResumeButton.TabIndex = 33;
            this.PauseResumeButton.UseVisualStyleBackColor = false;
            this.PauseResumeButton.Click += new System.EventHandler(this.PauseResumeButton_Click);
            // 
            // EvolverStatusGroup
            // 
            this.EvolverStatusGroup.Controls.Add(this.NumberOfIterationsTextBox);
            this.EvolverStatusGroup.Controls.Add(this.NumberOfIterationsLabel);
            this.EvolverStatusGroup.Controls.Add(this.CreatedPopulationSizeTextBox);
            this.EvolverStatusGroup.Controls.Add(this.CreatedPopulationSizeLabel);
            this.EvolverStatusGroup.Location = new System.Drawing.Point(597, 57);
            this.EvolverStatusGroup.Name = "EvolverStatusGroup";
            this.EvolverStatusGroup.Size = new System.Drawing.Size(192, 103);
            this.EvolverStatusGroup.TabIndex = 0;
            this.EvolverStatusGroup.TabStop = false;
            this.EvolverStatusGroup.Text = "Genetic Program Status";
            // 
            // NumberOfIterationsTextBox
            // 
            this.NumberOfIterationsTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.NumberOfIterationsTextBox.Location = new System.Drawing.Point(8, 76);
            this.NumberOfIterationsTextBox.Name = "NumberOfIterationsTextBox";
            this.NumberOfIterationsTextBox.ReadOnly = true;
            this.NumberOfIterationsTextBox.Size = new System.Drawing.Size(176, 22);
            this.NumberOfIterationsTextBox.TabIndex = 0;
            this.NumberOfIterationsTextBox.TabStop = false;
            // 
            // NumberOfIterationsLabel
            // 
            this.NumberOfIterationsLabel.Location = new System.Drawing.Point(8, 60);
            this.NumberOfIterationsLabel.Name = "NumberOfIterationsLabel";
            this.NumberOfIterationsLabel.Size = new System.Drawing.Size(176, 16);
            this.NumberOfIterationsLabel.TabIndex = 0;
            this.NumberOfIterationsLabel.Text = "# of Crossover Iterations";
            // 
            // CreatedPopulationSizeTextBox
            // 
            this.CreatedPopulationSizeTextBox.BackColor = System.Drawing.Color.Gainsboro;
            this.CreatedPopulationSizeTextBox.Location = new System.Drawing.Point(8, 34);
            this.CreatedPopulationSizeTextBox.Name = "CreatedPopulationSizeTextBox";
            this.CreatedPopulationSizeTextBox.ReadOnly = true;
            this.CreatedPopulationSizeTextBox.Size = new System.Drawing.Size(176, 22);
            this.CreatedPopulationSizeTextBox.TabIndex = 0;
            this.CreatedPopulationSizeTextBox.TabStop = false;
            // 
            // CreatedPopulationSizeLabel
            // 
            this.CreatedPopulationSizeLabel.Location = new System.Drawing.Point(8, 18);
            this.CreatedPopulationSizeLabel.Name = "CreatedPopulationSizeLabel";
            this.CreatedPopulationSizeLabel.Size = new System.Drawing.Size(176, 16);
            this.CreatedPopulationSizeLabel.TabIndex = 0;
            this.CreatedPopulationSizeLabel.Text = "# of Programs Created";
            // 
            // WarningLabel
            // 
            this.WarningLabel.ForeColor = System.Drawing.Color.DarkRed;
            this.WarningLabel.Location = new System.Drawing.Point(400, 564);
            this.WarningLabel.Name = "WarningLabel";
            this.WarningLabel.Size = new System.Drawing.Size(191, 41);
            this.WarningLabel.TabIndex = 0;
            // 
            // TitleLabel
            // 
            this.TitleLabel.Font = new System.Drawing.Font("Verdana", 9F, System.Drawing.FontStyle.Bold, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.TitleLabel.Location = new System.Drawing.Point(4, 4);
            this.TitleLabel.Name = "TitleLabel";
            this.TitleLabel.Size = new System.Drawing.Size(192, 44);
            this.TitleLabel.TabIndex = 34;
            this.TitleLabel.Text = "Ant Genetic Program           by Michael Lalena   http://www.lalena.com/";
            // 
            // AntEvolveProgramTab
            // 
            this.BackColor = System.Drawing.SystemColors.Control;
            this.Controls.Add(this.TitleLabel);
            this.Controls.Add(this.WarningLabel);
            this.Controls.Add(this.EvolverStatusGroup);
            this.Controls.Add(this.PauseResumeButton);
            this.Controls.Add(this.MapGroup);
            this.Controls.Add(this.CurrentAverageResultGroup);
            this.Controls.Add(this.CurrentBestResultGroup);
            this.Controls.Add(this.InterpreterGroup);
            this.Controls.Add(this.StartGPButton);
            this.Controls.Add(this.CrossoverGroup);
            this.Controls.Add(this.FitnessCalculationLabel);
            this.Controls.Add(this.InitialPopulationGroup);
            this.Controls.Add(this.InstructionSetLabel);
            this.Controls.Add(this.InstructionSetList);
            this.Font = new System.Drawing.Font("Verdana", 9F, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, ((byte)(0)));
            this.ForeColor = System.Drawing.Color.Black;
            this.Name = "AntEvolveProgramTab";
            this.Size = new System.Drawing.Size(800, 616);
            this.InitialPopulationGroup.ResumeLayout(false);
            this.InitialPopulationGroup.PerformLayout();
            this.FitnessCalculationLabel.ResumeLayout(false);
            this.FitnessCalculationLabel.PerformLayout();
            this.CrossoverGroup.ResumeLayout(false);
            this.CrossoverGroup.PerformLayout();
            this.InterpreterGroup.ResumeLayout(false);
            this.InterpreterGroup.PerformLayout();
            this.CurrentBestResultGroup.ResumeLayout(false);
            this.CurrentBestResultGroup.PerformLayout();
            this.CurrentAverageResultGroup.ResumeLayout(false);
            this.CurrentAverageResultGroup.PerformLayout();
            this.MapGroup.ResumeLayout(false);
            this.EvolverStatusGroup.ResumeLayout(false);
            this.EvolverStatusGroup.PerformLayout();
            this.ResumeLayout(false);

		}
		#endregion

        /// <summary>
        /// Private reference to the main form which gets us access to the other tabs.
        /// </summary>
        private AntForm antForm;
        /// <summary>
        ///The main form which gets us access to the other tabs.
        /// </summary>
        public AntForm AntForm
        {
            set
            {
                antForm = value;
            }
            get
            {
                return antForm;
            }
        }

        /// <summary>
        /// Used by the other tabs to get the currently best evolved program.
        /// If no program has been evolved yet, a single NoOp program is returned.
        /// </summary>
        /// <returns>The best program that has been evolved.</returns>
        public Program GetEvolvedProgram()
        {
            return evolver.BestProgram;
        }

        /// <summary>
        /// Based on the Tokens enum, populate the checked list on this tab with a list of ant program instructions.
        /// </summary>
		private void PopulateInstructionSetList()
		{
			foreach ( string instruction in Enum.GetNames(typeof(Tokens.Instruction)))
			{
				InstructionSetList.Items.Add(Tokens.DisplayableInstruction(instruction), InstructionIsOnByDefault(Tokens.StringToInstruction(instruction)));
			}
		}

        /// <summary>
        /// Start the Genetic Program based on the options provided on this tab.
        /// </summary>
        /// <param name="sender">The button object that generated this event.</param>
        /// <param name="e">Event arguments.</param>
		private void StartGPButton_Click(object sender, System.EventArgs e)
		{
            StartGPButton.Enabled = false;
            PauseResumeButton.Enabled = true;
            PauseResumeButton.Text = "Pause Genetic Program";

            if (evolver != null)
            {
                evolver.IterationCompleted -= new Evolver.IterationCompletedEventHandler(evolver_IterationCompleted);
            }

            evolver = new Evolver();
            evolver.IterationCompleted += new Evolver.IterationCompletedEventHandler(evolver_IterationCompleted);
            SetupEvolver();
            ThreadPool.QueueUserWorkItem(new WaitCallback(evolver.EvolveSolution));
        }

        /// <summary>
        /// Set up the GeneticProgramOptions for the Evolver object.
        /// </summary>
        private void SetupEvolver()
        {
            GeneticProgramOptions options = GetOptions();

			for (int i = 0; i < InstructionSetList.Items.Count; i++)
			{
                if (InstructionSetList.GetItemChecked(i))
                {
                    string instructionName = InstructionSetList.Items[i].ToString();
                    instructionName = instructionName.Replace(")", "");
                    instructionName = instructionName.Replace("(", "");
                    instructionName = instructionName.Replace(" ", "");
                    instructionName = instructionName.Replace(";", "");

                    Tokens.Instruction instruction = Tokens.StringToInstruction(instructionName);

                    if (Tokens.IsLeafNode(instruction))
                    {
                        options.LeafInstructionSet.Add(instruction);
                    }
                    else
                    {
                        options.NonLeafInstructionSet.Add(instruction);
                    }
                }
			}

            options.Map = GetSelectedMap(true);

            evolver.Options = options;
		}

        /// <summary>
        /// Creates a GeneticProgramOptions object and sets the options based on the values on this tab.
        /// This function is also reused by the AntSimulatorTab just to get the fitness calculation parameters.
        /// </summary>
        /// <returns>The options for the Genetic Program.</returns>
        public GeneticProgramOptions GetOptions()
        {
            GeneticProgramOptions options = new GeneticProgramOptions();
			options.FitenssProgramComplexityGrouping = Convert.ToInt32(ProgramComplexityGroupingTextBox.Text);
			options.FitnessProgramComplexity = Convert.ToInt32(ProgramComplexityValueTextBox.Text);
			options.FitnessRemainingFood = Convert.ToInt32(RemainingFoodValueTextBox.Text);
			options.FitnessUndiscoveredFood = Convert.ToInt32(UndiscoveredFoodValueTextBox.Text);
            options.FitnessWaterRemaining = Convert.ToInt32(WaterRemainingValueTextBox.Text);
            options.FitnessNumberOfTurns = Convert.ToInt32(SolutionSpeedValueTextBox.Text);
			options.PercentMutation = Convert.ToInt32(PercentMutateTextBox.Text);
			options.PercentGreedyMutation = Convert.ToInt32(PercentGreedyMutateTextBox.Text);
			options.PopulationSize = Convert.ToInt32(PopulationSizeTextBox.Text);
			options.TournamentSize = Convert.ToInt32(TournamentSizeTextBox.Text);
			options.PercentLeafNode = Convert.ToInt32(PercentLeafNodeTextBox.Text);
            options.MaximumProgramSize = Convert.ToInt32(MaximumProgramSizeTextBox.Text);
			options.NumberOfAnts = Convert.ToInt32(NumberOfAntsTextBox.Text);
			options.NumberOfTurnsPerAnt = Convert.ToInt32(NumberOfTurnsPerAntTextBox.Text);
			options.PheromoneStrength = Convert.ToInt32(PheromoneStrengthTextBox.Text);
			options.PheromoneDissipationSpeed = Convert.ToInt32(PheromoneSpreadSpeedTextBox.Text);
			options.WaitForHelpThisTurnPercent = Convert.ToInt32(WaitForHelpPercentTextBox.Text);
            options.AutomaticallyDropFood = AutoDropFoodCheckBox.Checked;

            return options;
        }

        /// <summary>
        /// Retrieves the map object for this Genetic Program.
        /// This function is also reused by the AntSimulatorTab so that the same map can be used when running a solution.
        /// </summary>
        /// <returns></returns>
        public Map GetMap()
        {
            return GetSelectedMap(false);
        }

        /// <summary>
        /// Get a map based on the CannedMapComboBox value.
        /// </summary>
        /// <param name="okToGetExternalMap">If true, it is OK to try to get the map from the Simluator tab.</param>
        /// <returns>The desired map referenced in CannedMapComboBox.</returns>
        private Map GetSelectedMap(bool okToGetExternalMap)
        {
            Map map = new Map();
            if (CannedMapComboBox.Text.Equals("Use Custom Map from Tab3"))
			{
                // Without this check, both tabs in the application could reference each other and get an infinite loop.
                if (okToGetExternalMap)
                {
                    map = AntForm.GetMapFromSimulatorTab();
                }
                else
                {
                    map = new Map(Map.CannedMaps.ThreeFood1Piles);
                }
			}
			else if ( CannedMapComboBox.Text.Equals( "Three Food(1) Piles" ))
			{
				map.LoadCannedMap( Map.CannedMaps.ThreeFood1Piles );
			}
			else if ( CannedMapComboBox.Text.Equals( "Six Weight Food(1,3,6) Piles" ))
			{
				map.LoadCannedMap( Map.CannedMaps.SixDifferentWeightFoodPiles );
			}
			else if ( CannedMapComboBox.Text.Equals( "Seven Food(6) Pieces" ))
			{
				map.LoadCannedMap( Map.CannedMaps.SevenFood6Pieces );
			}
			else if ( CannedMapComboBox.Text.Equals( "Ten Food(9) Pieces" ))
			{
				map.LoadCannedMap( Map.CannedMaps.TenFood9Pieces );
			}
            else if (CannedMapComboBox.Text.Equals("One Food(1) Pile With River"))
            {
                map.LoadCannedMap(Map.CannedMaps.OneFood1PileWithRiver);
            }
            else if (CannedMapComboBox.Text.Equals("Three Food(1) Piles River Trees"))
            {
                map.LoadCannedMap(Map.CannedMaps.ThreeFood1PileWithRiverAndTree);
            }
			else if ( CannedMapComboBox.Text.Equals( "Six Food(5) Pieces Wall/River" ))
			{
				map.LoadCannedMap( Map.CannedMaps.SixFood5WithWallAndRiver );
			}

            return map;
        }

        /// <summary>
        /// Should a given instruction be checked in the instruction set list by default when this program is started.
        /// </summary>
        /// <param name="instruction">The instruction to check.</param>
        /// <returns>True if the instruction should be checked.</returns>
        private static bool InstructionIsOnByDefault(Tokens.Instruction instruction)
        {
            if ((instruction == Tokens.Instruction.IfMoveToAdjacentFood) ||
                (instruction == Tokens.Instruction.IfCarryingFood) ||
                (instruction == Tokens.Instruction.MoveToNest) ||
                (instruction == Tokens.Instruction.IfMoveToPheromoneAwayFromNest) ||
                (instruction == Tokens.Instruction.IfFoodHere) ||
                (instruction == Tokens.Instruction.MoveQuasiRandom) ||
                (instruction == Tokens.Instruction.NoOp) ||
                (instruction == Tokens.Instruction.ReleasePheromone) || 
                (instruction == Tokens.Instruction.TwoStatements) ||
                (instruction == Tokens.Instruction.PickUpFood))
            {
                return true;
            }
            return false;
        }

        /// <summary>
        /// Event was raised by the evolver indicating that a new best solution was found.
        /// Events are also raised every 10 iterations to track progress.
        /// </summary>
        /// <param name="sender">The Evolver object that generated this event.</param>
        /// <param name="e">Event Arguments.</param>
        public void evolver_IterationCompleted(object sender, IterationCompleteEventArgs e)
        {
            if (this.InvokeRequired)
            {
                try
                {
                    this.Invoke(new UpdateTourEventHandler(UpdateFitnessDisplay), new object[] { sender, e });
                    Thread.Sleep(20);   // Due to CPU load from this program, need to give time for paint to occur.
                }
                catch (Exception)
                {
                }
            }

            // Either on GUI thread (not a real possiblity) or we are running in IE and a Security Exception
            // was thrown, so just try to do the update without invoking on the GUI thread.
            // Not a real good idea, but we don't have much of a choice.
            UpdateFitnessDisplay(sender, e);
            Thread.Sleep(20);   // Due to CPU load from this program, need to give time for paint to occur.
        }

        /// <summary>
        /// The delegate used to invoke back on the GUI thread to display the current Genetic Program status.
        /// </summary>
        /// <param name="sender">The Evolver object that started this event.</param>
        /// <param name="e">Event Arguments</param>
        public delegate void UpdateTourEventHandler(Object sender, IterationCompleteEventArgs e);

        /// <summary>
        /// Updates the Genetic Program status text boxes on this tab.
        /// </summary>
        /// <param name="sender">The Evolver object that started this event.</param>
        /// <param name="e">Event Arguments</param>
        public void UpdateFitnessDisplay(object sender, IterationCompleteEventArgs e)
        {
            if (e.NumberOfProgramsCreated < Convert.ToInt32(PopulationSizeTextBox.Text))
            {
                this.CreatedPopulationSizeTextBox.Text = e.NumberOfProgramsCreated.ToString() + "/" + PopulationSizeTextBox.Text;
            }
            else
            {
                this.CreatedPopulationSizeTextBox.Text = e.NumberOfProgramsCreated.ToString();
            }
            this.NumberOfIterationsTextBox.Text = e.Iteration.ToString();
            if (e.Program != null)
            {
                this.BestProgramFitnessTextBox.Text = e.Program.Fitness.ToString();
                this.BestProgramFoodNotCollectedTextBox.Text = e.Program.FoodLeft.ToString();
                this.BestProgramFoodNotDiscoveredTextBox.Text = e.Program.FoodNotFound.ToString();
                this.BestProgramWaterRemainingTextBox.Text = e.Program.WaterRemaining.ToString();
                this.BestProgramSizeTextBox.Text = e.Program.ProgramSize.ToString();
                this.BestProgramNumberOfTurnsTextBox.Text = e.Program.NumberOfTurns.ToString();
                this.AverageProgramFitnessTextBox.Text = e.AverageFitness.ToString();
                this.AverageProgramSizeTextBox.Text = e.AverageProgramSize.ToString();
            }
            else
            {
                this.BestProgramFitnessTextBox.Text = "";
                this.BestProgramFoodNotCollectedTextBox.Text = "";
                this.BestProgramFoodNotDiscoveredTextBox.Text = "";
                this.BestProgramWaterRemainingTextBox.Text = "";
                this.BestProgramSizeTextBox.Text = "";
                this.BestProgramNumberOfTurnsTextBox.Text = "";
                this.AverageProgramFitnessTextBox.Text = "";
                this.AverageProgramSizeTextBox.Text = "";
            }
        }

        /// <summary>
        /// When running in IE, call this when the user leaves the page
        /// to make sure the background threads don't keep running.
        /// </summary>
        public void StopThreads()
        {
            if (evolver != null)
            {
                evolver.Running = false;
                evolver = null;
            }
        }

        /// <summary>
        /// Called when the user clicks the PauseResumeButton.
        /// </summary>
        /// <remarks>
        /// This button is disabled if the user hasn't started the Genetic Program yet.
        /// If the Genetic Program is running, this will pause the evolver.
        /// If the Genetic Program has been paused, this will resume the evolver.
        /// Note: When the evolver is resumed, it will use any updated options on this tab.
        /// The user can change the population size, add or remove ants, and new instructions to the list...
        /// </remarks>
        /// <param name="sender">The button object that generated this event</param>
        /// <param name="e">Event Arguments.</param>
        private void PauseResumeButton_Click(object sender, EventArgs e)
        {
            if ( PauseResumeButton.Text.Equals("Pause Genetic Program"))
            {
                evolver.Running = false;
                PauseResumeButton.Text = "Resume Genetic Program";
                StartGPButton.Enabled = true;
                StartGPButton.Text = "Restart Genetic Program";
            }
            else
            {
                StartGPButton.Enabled = false;
                PauseResumeButton.Enabled = true;
                PauseResumeButton.Text = "Pause Genetic Program";

                SetupEvolver();
                ThreadPool.QueueUserWorkItem(new WaitCallback(evolver.ResumeSolution));
            }
        }

        private void CannedMapComboBox_SelectedIndexChanged(object sender, EventArgs e)
        {
            MapArea.DrawMap(GetSelectedMap(true));
        }

        /// <summary>
        /// Evaluate the value in the text box.
        /// Make sure its value is an integer, and its value is between the min and max value.
        /// </summary>
        /// <param name="textBox">The text box to evaluate.</param>
        /// <param name="minValue">The minimum acceptable value for the text box.</param>
        /// <param name="maxValue">The maximum acceptable value for the text box.</param>
        private void TestField(TextBox textBox, int minValue, int maxValue)
        {
            int number = minValue;

            try
            {
                number = int.Parse(textBox.Text);

                if (number < minValue)
                {
                    number = minValue;
                }
                if (number > maxValue)
                {
                    number = maxValue;
                }
            }
            catch (System.OverflowException)
            {
                number = maxValue;
            }
            catch (Exception)
            {
            }
            textBox.Text = number.ToString();
        }

        private void OptionChanged(object sender, EventArgs e)
        {
            WarningLabel.Text = ""; // reset the warning label. All checks below will append to the label.

            TestField(this.PopulationSizeTextBox, 10, 10000);
            TestField(this.MaximumProgramSizeTextBox, 3, 1000);
            TestField(this.PercentLeafNodeTextBox, 0, 100);
            TestField(this.MaximumProgramSizeTextBox, 3, 999);
            TestField(this.TournamentSizeTextBox, 4, 500);
            TestField(this.PercentMutateTextBox, 0, 100);
            TestField(this.PercentGreedyMutateTextBox, 0, 100);
            TestField(this.RemainingFoodValueTextBox, 0, 1000000000);
            TestField(this.UndiscoveredFoodValueTextBox, 0, 1000000000);
            TestField(this.WaterRemainingValueTextBox, 0, 1000000000);
            TestField(this.ProgramComplexityValueTextBox, 0, 1000000000);
            TestField(this.ProgramComplexityGroupingTextBox, 0, 10000);
            TestField(this.SolutionSpeedValueTextBox, 0, 1000000000);
            TestField(this.NumberOfAntsTextBox, 1, 1000);
            TestField(this.PheromoneStrengthTextBox, 100, 1000);
            TestField(this.PheromoneSpreadSpeedTextBox, 0, 100);
            TestField(this.WaitForHelpPercentTextBox, 0, 100);
            TestField(this.NumberOfTurnsPerAntTextBox, 100, 10000);

            // ProgramSize must be odd
            int number = int.Parse(this.MaximumProgramSizeTextBox.Text);
            if (number % 2 == 0)
            {
                number++;
                this.MaximumProgramSizeTextBox.Text = number.ToString();
            }
        }
	}
}
