//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: IterationCompleteEventArgs.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections.Generic;
using System.Text;

namespace AntGP
{
    /// <summary>
    /// Event Arguments used by the Evolver to indicate that it has completed an iteration of the Genetic Program.
    /// </summary>
    public class IterationCompleteEventArgs
    {
        /// <summary>
        /// Default Constructor
        /// </summary>
        public IterationCompleteEventArgs()
        {
        }

        /// <summary>
        /// Constructor that provides the event arguments.
        /// </summary>
        /// <param name="program">The best program found by the Genetic Program.</param>
        /// <param name="averageFitness">The average fitness of all programs in the population.</param>
        /// <param name="averageProgramSize">The average size of all programs in the population.</param>
        /// <param name="numberOfProgramsCreated">The total number of programs in the population.</param>
        /// <param name="iteration">How many time crossover has been performed on the population.</param>
        public IterationCompleteEventArgs(Program program, long averageFitness, int averageProgramSize, int numberOfProgramsCreated, int iteration)
		{
            this.program = program;
            this.averageFitness = averageFitness;
            this.averageProgramSize = averageProgramSize;
            this.numberOfProgramsCreated = numberOfProgramsCreated;
            this.iteration = iteration;
		}

        /// <summary>
        /// Private for the best program found by the evolver.
        /// </summary>
        private Program program;
        /// <summary>
        /// The best program found by the evolver.
        /// </summary>
        public Program Program
        {
            get
            {
                return program;
            }
        }

        /// <summary>
        /// Private for the average fitness of all programs in the population.
        /// </summary>
        private long averageFitness;
        /// <summary>
        /// The average fitness of all programs in the population.
        /// </summary>
        public long AverageFitness
        {
            get
            {
                return averageFitness;
            }
        }

        /// <summary>
        /// Private for the average size of all programs in the population.
        /// </summary>
        private long averageProgramSize;
        /// <summary>
        /// The average size of all programs in the population.
        /// </summary>
        public long AverageProgramSize
        {
            get
            {
                return averageProgramSize;
            }
        }

        /// <summary>
        /// Private for the total number of programs in the population.
        /// </summary>
        private int numberOfProgramsCreated;
        /// <summary>
        /// The total number of programs in the population.
        /// </summary>
        public int NumberOfProgramsCreated
        {
            get
            {
                return numberOfProgramsCreated;
            }
        }

        /// <summary>
        /// Private for how many time crossover has been performed on the population.
        /// </summary>
        private int iteration;
        /// <summary>
        /// How many time crossover has been performed on the population.
        /// </summary>
        public int Iteration
        {
            get
            {
                return iteration;
            }
        }
    }
}
