//////////////////////////////////////////////////////////////////////////////////////////////////
// File Name: SimulatorControl.cs
//      Date: 06/01/2006
// Copyright (c) 2006 Michael LaLena. All rights reserved.  (www.lalena.com)
// Permission to use, copy, modify, and distribute this Program and its documentation,
//  if any, for any purpose and without fee is hereby granted, provided that:
//   (i) you not charge any fee for the Program, and the Program not be incorporated
//       by you in any software or code for which compensation is expected or received;
//   (ii) the copyright notice listed above appears in all copies;
//   (iii) both the copyright notice and this Agreement appear in all supporting documentation; and
//   (iv) the name of Michael LaLena or lalena.com not be used in advertising or publicity
//          pertaining to distribution of the Program without specific, written prior permission. 
///////////////////////////////////////////////////////////////////////////////////////////////////
using System;
using System.Collections;
using System.ComponentModel;
using System.Drawing;
using System.Windows.Forms;

namespace AntGP
{
    /// <summary>
    /// This control displays the ant program being executed.
    /// </summary>
	public class SimulatorControl : System.Windows.Forms.UserControl
	{
        /// <summary>
        /// Default constructor.
        /// </summary>
		public SimulatorControl()
		{
			// This call is required by the Windows Form Designer.
			InitializeComponent();

            // Two clicks will place two items.
			this.SetStyle( ControlStyles.StandardDoubleClick, false );

            // This will eliminate the flickering.
			this.SetStyle( ControlStyles.UserPaint, true );
			this.SetStyle( ControlStyles.DoubleBuffer, true );
			this.SetStyle( ControlStyles.AllPaintingInWmPaint, true );
		}

		/// <summary>
		/// Clean up any resources being used.
		/// </summary>
		protected override void Dispose( bool disposing )
		{
			base.Dispose( disposing );
		}

		#region Designer generated code
		/// <summary>
		/// Required method for Designer support - do not modify
		/// the contents of this method with the code editor.
		/// </summary>
		private void InitializeComponent()
		{
			// 
			// SimulatorControl
			// 
			this.BackColor = System.Drawing.Color.White;
			this.Name = "SimulatorControl";

		}
		#endregion

        /// <summary>
        /// Draw the map image on this control
        /// </summary>
        /// <param name="map">The map to draw.</param>
		public void DrawMap(Map map)
		{
			this.BackgroundImage = map.DrawMap(this.Size);
		}

        /// <summary>
        /// Draw the map image with ants on this control.
        /// </summary>
        /// <param name="map">The map to draw.</param>
        /// <param name="ants">The ants to draw.</param>
        public void DrawMap(Map map, Colony ants)
        {
            this.BackgroundImage = map.DrawMap(this.Size, ants);
        }
	}
}